#include <stdlib.h>
#include <avr/delay.h>

#define _1WIRE_C


// configure your port here
// set pin mask   
#define wmask     0x00         
// set 1-wire pin for output
#define wOut      DDRC |= wmask
// set 1-wire pin for input (bus released)     
#define wIn       DDRC &= ~wmask
// set 1-wire bus high     
#define wH        PORTC |= wmask
// set 1-wire bus low     
#define wL        PORTC &= ~wmask
// get 1-wire bus input (bus released)   
#define wR        PINC & wmask   

#include "1Wire.h"


void uDelay(u8 count)
{
    while(count--);                 
}

void iDelay(unsigned int count)
{
    while(count--); 
}

void wTxbit (u8 bit)
{
   wOut;wL;
   if (bit)
   {
      DELAY_10US;    // 10.58us
	  wH;
        
	  DELAY_45US;   // 61.82us
	  DELAY_10US;  
   }
   else
   {
      DELAY_60US;     // 61.82us
                // better releash the bus
      // DELAY_10US;      // 10.58us
	  DELAY_10US;  
	  wH;
   }
   // wH; // by Gerrit */
}

u8 wRxbit (void)
{
   u8 data;
/*
   wOut;wL;
   DELAY__1US;                    // 10.58us
   wIn;
   DELAY_15US;   
   data=wR;
   DELAY_45US;       // 61.82us
*/


   wOut;wL;
   DELAY__1US;         // 10.58us
   wIn;
   DELAY_10US;                 // Lesen innerhalb der ersten 15 uS
   data=wR;
   DELAY_60US;       // 61.82us
  
   return data;
}
 

void wTxbyte (u8 data)
{
    u8 i;

    for (i=0; i<8; i++)
    {
        wTxbit (data & 0x01);
        data>>=1;
    }
}
 

u8 wRxbyte (void)
{
    u8 i, bit, data=0;

    data=0x00;
    for (i=0; i<8; i++)
    {
        bit=wRxbit();
        data|=bit<<i;
    }

    return data;
} 

u8 Reset1Wire (void)
{
    u8 present;

    wOut;wL;
    DELAY480US;
  	wIn;
    DELAY_15US;
    present=wR;
    DELAY410US;  
	
    return present;
}

void GetT(void)
{
	u8 get[10];
	char debugOut[80];
	char temp_lsb,temp_msb;
	int k;
	char temp_f,temp_c;
	char retVal, convflag, timeout = 0;

	retVal = Reset1Wire();

	debug(">>Reset device\r\n");

	if (retVal==0)
	{
	  debug(">>no device found!\r\n\0");
	  return;
	} else
	{
	  debug(">>device found!\r\n\0");
	}

	debug(">>Skip Rom \r\n\0");

	wTxbyte(0xCC); //Skip ROM

	debug(">>Start conversation\r\n\0");

	wTxbyte(0x44); // Start Conversion
    convflag=wRxbyte(); timeout=0;
    // Poll for convertion complete or timeout
	while (!convflag && timeout<200)
	{
   	 DELAY_60US;DELAY_60US;DELAY_60US;
     convflag=wRxbyte(); // Read a byte
   	 timeout++;
	}
	debug(">>Reset device 2\r\n\0");

	retVal = Reset1Wire();
	if (retVal==0)
	{
	  debug(">>no device found!\r\n\0");
	  return;
	} else
	{
	  debug(">> device found!\r\n\0");
	}

	debug(">>Skip Rom \r\n\0");
	wTxbyte(0xCC); // Skip ROM
    SIO_putStr(">>Read Scratchpad \r\n\0");
	wTxbyte(0xBE); // Read Scratch Pad
	for (k=0;k<9;k++){get[k]=wRxbyte();}
	SIO_putStr(">>Read Scratchpad.Done \r\n\0");

	sprintf(debugOut,"%#X %#X %#X %#X %#X %#X %#X %#X %#X \r\n\0",get[8],get[7],get[6],get[5],get[4],get[3],get[2],get[1],get[0]);
    SIO_putStr(debugOut);
	SIO_putStr(">>Calculate Temperature \r\n\0");
	SIO_putStr(">>check crc ");
	if ( crc8( &get[0], 9) )
	{
		SIO_putStr("failed \r\n\0");
	} else SIO_putStr("ok \r\n\0");
	temp_msb = get[1]; // Sign byte + lsbit
	temp_lsb = get[0]; // Temp data plus lsb
	if (temp_msb <= 0x80) { temp_lsb = (temp_lsb/2);} // shift to get whole degree
	temp_msb = temp_msb & 0x80; // mask all but the sign bit

	if (temp_msb >= 0x80) {temp_lsb = (~temp_lsb)+1;} // twos complement
	if (temp_msb >= 0x80) {temp_lsb = (temp_lsb/2);}// shift to get whole degree
	if (temp_msb >= 0x80) {temp_lsb = ((-1)*temp_lsb);} // add sign bit

	sprintf(debugOut,"%d Celsius\r\n\0", (int) temp_lsb ); // print temp. C
	SIO_putStr(debugOut);
	

}

#define _DEBUG 0
void debug(char *debugStr)
{
		if (_DEBUG)	SIO_putStr(debugStr);
}



